package Two;

use warnings;
use strict;

our $VERSION = '2.00';

sub dummy {
    print "Two dummy\n";
}

1;

__END__

=head1 NAME

One - The crappy ol' One module

=head1 SYNOPSIS

    use One;
    ...

=head1 DESCRIPTION

This module does something, but I forget what
